# Generated by Django 4.2.1 on 2023-05-23 11:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='MailchimpSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_key', models.CharField(blank=True, help_text='Mailchimp API Key ', max_length=50, null=True,
                                             verbose_name='Mailchimp API Key')),
                ('default_audience_id',
                 models.CharField(blank=True, help_text='Default Mailchimp Audience Id', max_length=100, null=True,
                                  verbose_name='Default Mailchimp Audience Id')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE,
                                              to='wagtailcore.site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.core.cache import cache
from mailchimp3 import MailChimp


class MailchimpApi:
    def __init__(self, api_key):
        self.client = None
        self.is_active = False

        self.init_api(api_key)

    def init_api(self, api_key):
        try:

            self.client = MailChimp(mc_api=api_key)
            self.is_active = True
        except Exception as e:
            return None

    def get_lists(self, fields='lists.id,lists.name'):
        cache_key = f"get-lists-{fields}"
        lists = cache.get(cache_key)

        if lists is None:
            try:
                result = self.client.lists.all(fields=fields, get_all=True)
                lists = result['lists']
                cache.set(cache_key, lists)
            except:
                return []

        return lists

    def get_merge_fields_for_list(self, list_id,
                                  fields="merge_fields.merge_id,"
                                         "merge_fields.tag,"
                                         "merge_fields.name,"
                                         "merge_fields.type,"
                                         "merge_fields.required,"
                                         "merge_fields.public,"
                                         "merge_fields.display_order,"
                                         "merge_fields.options,"
                                         "merge_fields.help_text",
                                  ):
        cache_key = f"get-merge-fields-{list_id}-{fields}"
        merge_fields = cache.get(cache_key)

        if merge_fields is None:
            try:
                result = self.client.lists.merge_fields.all(list_id=list_id, get_all=True, fields=fields)
                merge_fields = result['merge_fields']
                cache.set(cache_key, merge_fields)
            except:
                return []

        return merge_fields

    def get_interest_categories_for_list(self, list_id,
                                         fields="categories.id,"
                                                "categories.title,"
                                                "categories.type,"
                                                "categories.display_order"):
        cache_key = f"get-categories-{list_id}-{fields}"
        categories = cache.get(cache_key)

        if categories is None:
            try:
                result = self.client.lists.interest_categories.all(list_id=list_id, get_all=True, fields=fields)
                categories = result['categories']
                cache.set(cache_key, categories)
            except:
                return []

        return categories

    def get_interests_for_interest_category(self, list_id, interest_category_id,
                                            fields="interests.id,"
                                                   "interests.name,"
                                                   "interests.display_order"):
        cache_key = f"get-interests-{list_id}-{interest_category_id}-{fields}"
        interests = cache.get(cache_key)

        if interests is None:
            try:
                result = self.client.lists.interest_categories.interests.all(list_id=list_id,
                                                                             category_id=interest_category_id,
                                                                             get_all=True,
                                                                             fields=fields)
                interests = result['interests']
                cache.set(cache_key, interests)
            except:
                return []

        return interests

    def get_interests_for_list(self, list_id):
        interest_categories = self.get_interest_categories_for_list(list_id=list_id)

        categories = []

        for category in interest_categories:
            category_id = category.get('id', '')

            interest_category = {
                "id": category_id,
                "title": category.get('title', ''),
                'type': category.get('type', '')
            }

            interests = self.get_interests_for_interest_category(list_id=list_id, interest_category_id=category_id)

            interest_category['interests'] = interests

            categories.append(interest_category)

        return categories

    def add_user_to_list(self, list_id, data):
        return self.client.lists.members.create(list_id=list_id, data=data)



from collections import OrderedDict

from django import forms
from django.utils.translation import gettext_lazy as _
from django_countries.fields import CountryField

from wagtailmailchimp.widgets import CustomSelect


class MailChimpForm(forms.Form):
    required_css_class = 'required'

    """
    MailChimp list-based form class.
    """

    def __init__(self, merge_fields, interest_categories=None, *args, **kwargs):
        """
        Initailizes the form instance, adding fields for specified
        MailChimp merge fields and interest categories.

        :param merge_fields: list of merge variable dictionaries.
        :param interest_categories: list of grouping dictionaries.
        """
        # Initialize the form instance.
        super(MailChimpForm, self).__init__(*args, **kwargs)

        # append compulsory email field
        merge_fields.insert(0, ({
            "tag": "EMAIL",
            "name": "Email Address",
            "help_text": "Your Email Address",
            "type": "email",
            "required": "true",
            "options": {"size": 100}
        }))

        # Add merge variable fields.
        for merge_field in merge_fields:
            for data in self.mailchimp_field_factory(merge_field).items():
                name, field = data
                self.fields.update({name: field})

        if interest_categories is not None:
            # Add grouping fields.
            for interest_category in interest_categories:
                name = "INTERESTS"
                field = self.mailchimp_interest_category_factory(interest_category)

                self.fields.update({name: field})

    def mailchimp_field_factory(self, merge_field):
        """
        Returns a form field instance for specified MailChimp merge Field.

        :param merge_field: merge field dictionary.
        :rtype: django.forms.Field.
        """
        fields = OrderedDict()
        mc_type = merge_field.get('type', None)
        name = merge_field.get('tag', '')
        visible = merge_field.get('public', True)
        mc_options = merge_field.get('options', {})
        kwargs = {
            'label': merge_field.get('name', None),
            'required': merge_field.get('required', True),
            'initial': merge_field.get('default_value', None),
            'help_text': merge_field.get('help_text', None)
        }

        if not visible:
            kwargs.update({'widget': forms.HiddenInput})
            fields.update({name: forms.CharField(**kwargs)})
            return fields

        if mc_type == 'email':
            kwargs.update({'max_length': mc_options.get('size', None)})
            fields.update({name: forms.EmailField(**kwargs)})

        if mc_type == 'text':
            kwargs.update({'max_length': mc_options.get('size', None)})
            fields.update({name: forms.CharField(**kwargs)})

        if mc_type == 'number':
            fields.update({name: forms.IntegerField(**kwargs)})

        if mc_type == 'radio':
            kwargs.update({
                'choices': ((x, x) for x in mc_options.get('choices', [])),
                'widget': forms.RadioSelect
            })
            fields.update({name: forms.ChoiceField(**kwargs)})

        if mc_type == 'dropdown':
            kwargs.update({
                'choices': ((x, x) for x in mc_options.get('choices', []))
            })
            fields.update({name: forms.ChoiceField(**kwargs)})

        if mc_type == 'date' or mc_type == 'birthday':
            fields.update({name: forms.DateField(**kwargs)})

        if mc_type == 'address':
            # Define keyword agruments for each charfield component.
            char_fields = [
                {
                    'name': '{0}[addr1]'.format(name),
                    'label': "Street Address",
                    'required': True,
                    'max_length': 70,
                    "placeholder": "Street Address"
                },
                {
                    'name': '{0}[addr2]'.format(name),
                    'label': "Address Line 2",
                    'required': True,
                    'max_length': 70,
                    "placeholder": "Address Line 2"
                },
                {
                    'name': '{0}[city]'.format(name),
                    'label': "City",
                    'required': True,
                    'max_length': 40,
                    'placeholder': "City"
                },
                {
                    'name': '{0}[state]'.format(name),
                    'label': "State/Province/Region",
                    'required': True,
                    'max_length': 20,
                    'placeholder': "State/Province/Region"

                },
                {
                    'name': '{0}[zip]'.format(name),
                    'label': 'Postal/Zip',
                    'required': True,
                    'max_length': 10,
                    'placeholder': "Postal/Zip"
                },
            ]

            # Add the address charfields.
            for kwargs in char_fields:
                field_name = kwargs.pop('name')
                placeholder = kwargs.pop('placeholder', None)

                if placeholder:
                    kwargs.update({'widget': forms.TextInput(attrs={'placeholder': placeholder})})

                fields.update({field_name: forms.CharField(**kwargs)})

            # Finally, add the address country field.
            name = '{0}-country'.format(name)
            fields.update({
                name: CountryField(label=_('Country'))
            })

        if mc_type == 'zip':
            kwargs.update({'max_length': mc_options.get('size', None)})
            fields.update({name: forms.CharField(**kwargs)})

        if mc_type == 'phone':
            kwargs.update({'max_length': mc_options.get('size', None)})
            fields.update({name: forms.CharField(**kwargs)})

        if mc_type == 'url' or mc_type == 'imageurl':
            kwargs.update({'max_length': mc_options.get('size', None)})
            fields.update({name: forms.URLField(**kwargs)})

        return fields

    def mailchimp_interest_category_factory(self, interest_category):

        """
        Returns form field instance for specified MailChimp grouping.

        :param interest_category: interest_category dictionary.
        :param selected_interests: selected_interests list.
        :rtype: django.forms.Field.
        """

        field_type = interest_category.get('type', None)
        title = interest_category.get('title', None)

        interests = interest_category.get('interests', [])
        choices = ((x['id'], x['name']) for x in interests)
        kwargs = {'label': title, 'choices': choices, 'required': False}

        if field_type == 'checkboxes':
            kwargs.update({'widget': forms.CheckboxSelectMultiple})
            return forms.MultipleChoiceField(**kwargs)

        if field_type == 'radio':
            kwargs.update({'widget': forms.RadioSelect})
            return forms.ChoiceField(**kwargs)

        if field_type == 'dropdown':
            kwargs.update({'widget': forms.Select})
            return forms.ChoiceField(**kwargs)

        if field_type == 'hidden':
            kwargs.update({'widget': forms.HiddenInput})
            return forms.ChoiceField(**kwargs)


class MailchimpIntegrationForm(forms.Form):
    def __init__(self, merge_fields=None, form_fields=None, *args, **kwargs):
        # Initialize the form instance.
        super(MailchimpIntegrationForm, self).__init__(*args, **kwargs)

        if merge_fields and form_fields:
            merge_fields.insert(0, ({
                "tag": "EMAIL",
                "name": "Email Address",
                "type": "email",
                "required": "true",
            }))

            for field in merge_fields:
                choices = [("", "-- Select field to merge--")]

                if field.get("type") == "email":
                    for form_field in form_fields:
                        if form_field.field_type == 'email':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "number":
                    for form_field in form_fields:
                        if form_field.field_type == 'number':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "url":
                    for form_field in form_fields:
                        if form_field.field_type == 'url':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "radio":
                    for form_field in form_fields:
                        if form_field.field_type == 'radio':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "dropdown":
                    for form_field in form_fields:
                        if form_field.field_type == 'dropdown':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "checkboxes":
                    for form_field in form_fields:
                        if form_field.field_type == 'checkboxes':
                            choices.append((form_field.clean_name, form_field.label))
                elif field.get("type") == "date" or field.get("type") == "birthday":
                    for form_field in form_fields:
                        if form_field.field_type == 'date':
                            choices.append((form_field.clean_name, form_field.label))
                else:
                    for form_field in form_fields:
                        if form_field.field_type == 'singleline' or form_field.field_type == "multiline":
                            choices.append((form_field.clean_name, form_field.label))

                kwargs = {
                    'label': field.get('name', None),
                    'required': field.get('required', True)
                }

                name = field.get("tag")

                self.fields.update({name: forms.ChoiceField(choices=choices, widget=CustomSelect, **kwargs)})
                self.fields[name].label = field.get("name")



from django.forms.widgets import Input, Select
from wagtail.models import Site

from .api import MailchimpApi


class CustomSelect(Select):
    def create_option(self, *args, **kwargs):
        option = super().create_option(*args, **kwargs)
        if not option.get('value'):
            option['attrs']['disabled'] = True

        return option


class MailchimpSubscriberOptinWidget(Input):
    input_type = 'checkbox'
    template_name = 'wagtailmailchimp/widgets/subscriber_optin_widget.html'

    def __init__(self, attrs=None, interest_categories=None, label=None, interests_field_name=None):
        super(MailchimpSubscriberOptinWidget, self).__init__()
        self.interest_categories = interest_categories
        self.interests_field_name = interests_field_name
        self.label = label

    def get_context(self, name, value, attrs):
        ctx = super(MailchimpSubscriberOptinWidget, self).get_context(name, value, attrs)
        ctx['widget']['interest_categories'] = self.interest_categories
        ctx['widget']['label'] = self.label
        ctx['widget']['interests_field_name'] = self.interests_field_name
        return ctx


class MailchimpAudienceListWidget(Select):
    def __init__(self, attrs=None, choices=()):
        super().__init__(attrs, choices)

        audience_lists = self.get_mailchimp_audience_lists()
        audience_choices = [("", "-- None --")]

        for audience in audience_lists:
            audience_choices.append([audience.get("id"), audience.get("name")])

        self.choices = audience_choices

    def get_mailchimp_audience_lists(self):
        from .models import MailchimpSettings

        current_site = Site.objects.get(is_default_site=True)

        mc_settings = MailchimpSettings.for_site(current_site)
        mailchimp = MailchimpApi(api_key=mc_settings.api_key)
        lists = mailchimp.get_lists()

        return lists
